program VelHub_B

  use sep
  use weighthrt
  use velsimp6
  use huber_reg_mod
  use huber_bfgs_b
  use chain_mask_hrt
  use identity_mod

  implicit none

  integer                          :: nt, nx, ns, iadj, n2, niter, stat
  integer                          :: mmax, l1, n3, verb, j, bound
  real                             :: gamd, gamx, eps, nu, mu, accuracy, low, up
  real                             :: o1,d1, o2,d2, o3, d3, x0,dx, s0,ds
  real, dimension (:), allocatable :: data, model, recons, l, u
  real, dimension (:), pointer     :: mask
  logical                          :: adj

  call sep_init ()                 ! start processing parameters

  call from_history ("n3", n3)
  call from_history ("o3", o3)
  call from_history ("d3", d3)
  call from_history (nt  , n2)
  call from_history ("o2", o2)
  call from_history ("d2", d2)
  call from_either  ("nx", nx, n2)
  call from_either  ("ns", ns, 2*nx)
  call from_history ("o1", o1)
  call from_history ("d1", d1)
  call from_either  ("s0", s0) 
  call from_either  ("ds", ds)  
  call from_either  ("x0", x0, o2) 
  call from_either  ("dx", dx, d2) 
  call from_param   ("adj",iadj,1) ; adj = (iadj == 1)

  if (adj) then 
     call to_history ("label2","slowness(sec/km)")
     n2 = ns; o2 = s0; d2 = ds
  else
     call to_history ("label2","distance(km)")
     n2 = nx; o2 = x0; d2 = dx
  end if


  call to_history ("o2", o2)
  call to_history ("d2", d2)
  call to_history ("n2", n2)
  call from_param ("verb",verb,2)
  call from_param ("niter", niter, 1)
  call from_param ("mmax",mmax,2)
  call from_param ("gamd",gamd,1.)
  call from_param ("gamx",gamx,1.)
  call from_param ("nu",nu,0.9)
  call from_param ("mu",mu,0.0001)
  call from_param ("eps",eps,0.)
  call from_param ("l1",l1,0)
  call from_param ("bound_type",bound,0)
  call from_param ("accuracy",accuracy,1e-8)
  call from_param ("lower",low,0.)
  call from_param ("upper",up,1.)

  if (exist_file ("recons")) then
     call to_history ("o1", o1,"recons")
     call to_history ("d1", d1,"recons")
     call to_history ("n1", nt,"recons")
     call to_history ("o2", x0,"recons")
     call to_history ("d2", dx,"recons")
     call to_history ("n2", nx,"recons")
     call to_history ("o3", o3,"recons")
     call to_history ("d3", d3,"recons")
     call to_history ("n3", n3,"recons")
     call to_history ("label2","distance(km)","recons")
     allocate(recons(nt*nx))
  end if

  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>> HRT in the Time Domain'
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>> HRT - parameters:'
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>> HRT - q0  =:',s0
  if (verb>0) write(0,*) '>> HRT - nq  =:',ns
  if (verb>0) write(0,*) '>> HRT - dq  =:',ds
  if (verb>0) write(0,*) '>> HRT - verb=:',verb
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>> HRT - gamd  =:',gamd
  if (verb>0) write(0,*) '>> HRT - gamx  =:',gamx
  if (verb>0) write(0,*) '>> HRT - niter =:',niter
  if (verb>0) write(0,*) '>> HRT - eps   =:',eps
  if (verb>0) write(0,*) '>> HRT - l1    =:',l1
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>> HRT - '
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'

  call sep_close()                ! end processing parameters

  allocate (data(nt*nx), model(nt*ns), l(nt*ns), u(nt*ns))
  
  if (exist_file("mask")) then
     allocate(mask(nt*nx))
  end if

  l=low
  u=up

  call velsimp6_init (o1,d1,x0,dx,s0,ds,nt,nx,ns)
  call huber_init_out(gamd,gamx,l1,eps,nt*ns,verb)

  do j=1,n3

     data=0.
     model=0.

     if (adj) then 

        data=0.
        model=0.

        if (verb>0) write(0,*) '>>>> HRT -- GATHER #',j,'/',n3

        if (exist_file("mask")) then
           mask=0.
           call sep_read(mask,"mask")
           call weight_init(mask)
           call sep_read (data)
           data=data*mask
           call huber_init_in(data)
           call lbfgs (chain_mask_hrt_lop,identity_lop,huberfct,hubergdt,model,niter,accuracy,bound,l,u,verb)
           call huber_close()
           call sep_write (model)

        else

           call sep_read (data)
           call huber_init_in(data)
           call lbfgs (velsimp6_lop,identity_lop,huberfct,hubergdt,model,niter,accuracy,bound,l,u,verb)
           call huber_close()
           call sep_write (model)
           
        end if

        if (exist_file("recons")) then

           stat = velsimp6_lop(.false.,.false.,model,data)           
           call sep_write(data,"recons")

        end if

     else

        call sep_read (model)
        stat = velsimp6_lop (.false.,.false.,model,data)
        call sep_write(data)

     end if

  end do

  deallocate (model,data)
  if (exist_file ("recons")) deallocate(recons)

  call exit (0)

end program VelHub_B
