program PRT_B

  use sep
  use parainv_under
  use parainv_over
  use parabolic
  
  ! time domain approach with Huber function
  use parasimp_mod
  use huber_reg_mod
  use huber_bfgs_b
  use identity_mod
 
  implicit none

  integer                          :: nk, nt, nx, nq, iadj, n2, stat
  integer                          :: weight, alias, inversion
  real                             :: eps,pi,fhi,flo,gamd,gamx
  real                             :: o1,d1,o2,d2,o3,d3,x0,dx,q0,dq
  real, dimension (:), allocatable :: data, model, recons, wout
  logical                          :: adj
  integer                          :: n3, j, verb, niter, l1

  integer                          :: bound
  real, dimension (:), allocatable :: l, u
  real                             :: accuracy, low, up

  call sep_init ()                 ! start processing parameters
  
  call from_history ("n3", n3)
  call from_history ("o3", o3)
  call from_history ("d3", d3)
  call from_history (nt  , n2)
  call from_history ("o2", o2)
  call from_history ("d2", d2)
  call from_either  ("nx", nx, n2)
  call from_either  ("nq", nq, 2*nx)
  call from_history ("o1", o1)
  call from_history ("d1", d1)
  call from_either  ("q0", q0) 
  call from_either  ("dq", dq)  
  call from_either  ("x0", x0, o2) 
  call from_either  ("dx", dx, d2)
  call from_param   ("weight",weight,0) 
  call from_param   ("verb",verb,0)  
  call from_param   ("alias",alias,0)
  call from_param   ("inversion",inversion,3)
  call from_param   ("adj",iadj,1) ; adj = (iadj == 1)
  call from_param   ("fhi",fhi,1/(2*d1))
  call from_param   ("flo",flo,0.)
  call from_param   ("gamd",gamd,1.)
  call from_param   ("gamx",gamx,0.)
  call from_param   ("eps",eps,1.)
  call from_param   ("l1",l1,0)
  call from_param   ("niter",niter,10)
  call from_param   ("bound_type",bound,0)
  call from_param   ("accuracy",accuracy,1e-8)
  call from_param   ("lower",low,0.)
  call from_param   ("upper",up,1.)

  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>> PRT in the Time or Fourier Domain'
  if (verb>0) write(0,*) '>> PRT - '
  if (verb>0) write(0,*) '>> PRT - '
  if (verb>0) write(0,*) '>> PRT - parameters:'
  if (verb>0) write(0,*) '>> PRT - '
  if (verb>0) write(0,*) '>> PRT - '
  if (verb>0) write(0,*) '>> PRT - q0  =:',q0
  if (verb>0) write(0,*) '>> PRT - nq  =:',nq
  if (verb>0) write(0,*) '>> PRT - dq  =:',dq
  if (verb>0) write(0,*) '>> PRT - verb=:',verb
  if (verb>0) write(0,*) '>> PRT - inv =:',inversion
  if (verb>0) write(0,*) '>> PRT - '
  if (verb>0) write(0,*) '>> PRT - '
  if (verb>0) then
     
     if (inversion.eq.1) then
        if (verb>0) write(0,*) '     >> PRT - Fourier Approach with Inversion' 
        if (verb>0) write(0,*) '     >> PRT - '
        if (verb>0) write(0,*) '     >> PRT - '     
     else if (inversion.eq.2) then  
        if (verb>0) write(0,*) '     >> PRT - Fourier Approach without Inversion'
        if (verb>0) write(0,*) '     >> PRT - '
        if (verb>0) write(0,*) '     >> PRT - '     
     else if (inversion.eq.3) then
        if (verb>0) write(0,*) '     >> PRT - Time domain Approach with Inversion'
        if (verb>0) write(0,*) '     >> PRT - '
        if (verb>0) write(0,*) '     >> PRT - '     
     end if 
     
     if (inversion.le.2) then
        if (verb>0) write(0,*) '     >> PRT - weight=:',weight
        if (verb>0) write(0,*) '     >> PRT - alias =:',alias
        if (verb>0) write(0,*) '     >> PRT - fhi   =:',fhi
        if (verb>0) write(0,*) '     >> PRT - flo   =:',flo
        if (verb>0) write(0,*) '     >> PRT - '
        if (verb>0) write(0,*) '     >> PRT - '     
     else
        if (verb>0) write(0,*) '     >> PRT - gamd  =:',gamd
        if (verb>0) write(0,*) '     >> PRT - gamx  =:',gamx
        if (verb>0) write(0,*) '     >> PRT - niter =:',niter
        if (verb>0) write(0,*) '     >> PRT - eps   =:',eps
        if (verb>0) write(0,*) '     >> PRT - l1    =:',l1
        if (verb>0) write(0,*) '     >> PRT - '
        if (verb>0) write(0,*) '     >> PRT - '     
     end if

  end if

  if (adj) then 
     call to_history ("label2"," q ")
     n2 = nq; o2 = q0; d2 = dq
  else
     call to_history ("label2","distance")
     n2 = nx; o2 = x0; d2 = dx
  end if
  
  
  call to_history ("o2", o2)
  call to_history ("d2", d2)
  call to_history ("n2", n2)

  if (exist_file ("recons")) then
     call to_history ("o1", o1,"recons")
     call to_history ("d1", d1,"recons")
     call to_history ("n1", nt,"recons")
     call to_history ("o2", x0,"recons")
     call to_history ("d2", dx,"recons")
     call to_history ("n2", nx,"recons")
     call to_history ("o3", o3,"recons")
     call to_history ("d3", d3,"recons")
     call to_history ("n3", n3,"recons")
     call to_history ("label2","distance(km)","recons")
     allocate(recons(nt*nx))
  end if

  if (exist_file ("wout")) then 
     nk=1
     do
       if (nk.ge.nt) exit
       nk=nk*2
     end do
     nk=nk-nk/2+1
     write(0,*) 'Found weight file'
     call to_history ("o1", 0,"wout")
     call to_history ("d1", 1/(2*nk*d1),"wout")
     call to_history ("n1", nk,"wout")
     call to_history ("o2", q0,"wout")
     call to_history ("d2", dq,"wout")
     call to_history ("n2", nq,"wout")
     call to_history ("n3", n3,"wout")
     call to_history ("o3", o3,"wout")
     call to_history ("d3", d3,"wout")
     call to_history ("label2","q s.km-2","wout")
     call to_history ("label1","Frequency (Hz)","wout")
     call to_history ("title","Weights","wout")
     allocate(wout(nk*nq))
  end if
    
  call sep_close()                ! end processing parameters
  
  allocate (data(nt*nx),model(nt*nq))
 
  if (inversion.eq.1) then

     if ( nq.ge.nx ) then
        if (verb>0) write(0,*) '              PRT >> Fourier Domain - UNDER DETERMINED PROBLEM = SLOW'
        call parainvunder_init (o1,d1,x0,dx,q0,dq,nt,nx,nq, &
        & eps,weight,alias,nk,fhi,flo,verb)   
     else
        if (verb>0) write(0,*) '              PRT >> Fourier Domain -  OVER DETERMINED PROBLEM = FAST'
        call parainvover_init (o1,d1,x0,dx,q0,dq,nt,nx,nq,eps,verb)
     end if
     
  else if(inversion.eq.2) then
     
     if (verb>0) write(0,*) '               PRT >> Fourier Domain - Adjoint operator only, no inversion'
     call parabolic_init (o1,d1,x0,dx,q0,dq,nt,nx,nq,alias,fhi,flo)
  
  else
     
     if (verb>0) write(0,*) '               PRT >> Time Domain - Quasi-Newton / L-BFGS '
     allocate (l(nt*nq), u(nt*nq))
     l=low
     u=up
     call parasimp_init(o1,d1,x0,dx,q0,dq,nt,nx,nq,verb)
     call huber_init_out(gamd,gamx,l1,eps,nt*nq,verb)

  end if 
  
  do j=1,n3

     data=0.
     model=0.
     wout=0.
     
     if (verb>0) write(0,*) '>>>> PRT -- GATHER #',j,'/',n3 

     if (adj) then  
        
        call sep_read(data)
        
        if (inversion.eq.1) then 
           
           if ( nq.ge.nx ) then
              if (exist_file ("wout")) then 
                 stat = parainvunder_lop (.true.,.false.,model,data,wout)
                 call sep_write(wout,"wout")
              else
                 stat = parainvunder_lop (.true.,.false.,model,data)
              end if
           else
              stat = parainvover_lop (.true.,.false.,model,data)
           end if
           
        else if (inversion.eq.2) then
           
           stat = parabolic_lop (.true.,.false.,model,data)
           
        else
           
           call huber_init_in(data)
           call lbfgs (parasimp_lop,identity_lop,huberfct,hubergdt,model,niter,accuracy,bound,l,u,verb)
           call huber_close()

        end if
        
        call sep_write (model)
        
        if (exist_file("recons")) then
           if (inversion.eq.1) then
              if ( nq.ge.nx ) then
                 stat = parainvunder_lop (.false.,.false.,model,data,wout)
              else
                 stat = parainvover_lop (.false.,.false.,model,data)           
              end if              
           else if (inversion.eq.2) then    
              stat = parabolic_lop (.false.,.false.,model,data)              
           else
              stat = parasimp_lop(.false.,.false.,model,data)
           end if
           
           call sep_write(data,"recons")
        
        end if
        
     else
        
        call sep_read (model)
        
        if (inversion.eq.1) then
           if ( nq.ge.nx ) then     
              stat = parainvunder_lop (adj,.false.,model,data)
           else
              stat = parainvover_lop (adj,.false.,model,data)
           end if
        else if (inversion.eq.2) then
           stat = parabolic_lop (adj,.false.,model,data)
        else
           stat = parasimp_lop (adj,.false.,model,data)
        end if

        call sep_write (data)
        
     end if
  
  end do
  
  deallocate (model,data)
  if (exist_file ("recons")) deallocate(recons)
  
  call exit (0)
  
end program PRT_B






