module ctoeplitz
  implicit none
  integer,                                private :: n
  complex*16,    dimension (:), allocatable, private :: a
contains
  subroutine ctoeplitz_init (n_in)
    integer, intent (in) :: n_in
    n = n_in
    allocate (a (n)); a(1) = cmplx(1.,0.)
  end subroutine ctoeplitz_init
  
  ! r - top row of toeplitz matrix
  ! g - right-hand side
  ! f - solution
  ! solve r.f = g  (f unknown)
  
  subroutine ctoeplitz_solve (r, g, f)
    complex*16, dimension (:), intent (in)  :: r, g
    complex*16, dimension (:), intent (out) :: f
    
    integer               :: i,j
    complex*16            :: e,c,w
    double precision      :: v
    complex*16            :: bot
    
    v=r(1)
 
    f(1) = g(1)/r(1)
    
    do j=2, n
       e = sum(a(j-1:1:-1)*conjg(r(2:j)))
       c = -e/v

       v = v + c*conjg(e)
       
       if (abs(v).le.0.0000000001) then
          write(0,*) 'v too small'  
          return   
       end if

       do i=2, (j+1)/2
          bot = a(j-i+1) + c*conjg(a(i))
          a(i) = a(i) + c*conjg(a(j-i+1))
          a(j-i+1) = bot
       end do
       a(j) = c
       
       w = sum(f(j-1:1:-1)*conjg(r(2:j)))
       c = (g(j)-w)/v
       
       f(1:j-1) = f(1:j-1) + c*conjg(a(j:2:-1))
       f(j) = c
       
    end do
  end subroutine ctoeplitz_solve
  
  subroutine ctoeplitz_close()
    deallocate (a)
  end subroutine ctoeplitz_close
  
end module ctoeplitz










