module huber_mod
  
  implicit none
  
  real,dimension(:) , pointer :: dath
  real              , private :: gamdh
  real              , private :: gamxh
  real              , private :: epsi
  integer           , private :: l1
  integer           , private :: verb
  logical, parameter, private :: T = .true., F = .false.
  
  
contains
  
  subroutine huber_init_out(d,m,e,l1_in,verb_in)
    
    real                    :: d,m,e
    integer                 :: l1_in, verb_in
    
    gamdh  = d
    gamxh  = m
    epsi   = e
    l1     = l1_in
    verb   = verb_in

    if (verb>1) then
       if (epsi.ne.0.) then
          if (l1.eq.0) write(0,*) '- Regularization with the Huber norm'
          if (l1.eq.1) write(0,*) '- Regularization with the Cauchy norm'
       end if
    end if
    
  end subroutine huber_init_out

  subroutine huber_init_in(datah)
    
    real,dimension(:)       :: datah
    
    allocate(dath(size(datah)))
    
    dath  = datah
    
  end subroutine huber_init_in
  
  function huberfct(oper,x,fct) result (stat)

    interface

       function oper( adj, add, x, dat) result (stat)
         integer             :: stat
         logical, intent(in) :: adj, add
         real, dimension(: ) :: x, dat
       end function oper

    end interface

    integer                            :: stat,i
    real, dimension( :),intent( in)    :: x             
    real,               intent(out)    :: fct      
    real, dimension(size(dath))        :: rd
    real, dimension(size(x))           :: rm
    real                               :: tmp

    rd = - dath

    stat  = oper(F,T,x,rd)               ! r_d = L x - data

    where(abs(rd).gt.gamdh)
       rd = abs(rd)-gamdh/2.0
    elsewhere
       rd = rd*rd/(2.0*gamdh)
    endwhere

    fct = sum(rd)

    if (epsi.ne.0.0) then

       tmp=0.
       rm = x 

       if (l1.eq.0) then

          ! Huber
          where(abs(rm).gt.gamxh)
             rm = abs(rm)-gamxh/2.0
          elsewhere
             rm = rm*rm/(2.0*gamxh)
          endwhere

          tmp = sum(rm)

       else

          ! write(0,*) maxval(rm),minval(rm)
          ! Cauchy
          do i=1,size(rm)         
             tmp = tmp + log(1+rm(i)**2/gamxh**2)             
          end do
!          write(0,1*) tmp

       end if

       fct = epsi*tmp + fct

    end if

  end function huberfct
  
  function hubergdt(oper,x,g) result(stat)
    
    interface
       function oper( adj, add, x, dat) result (stat)
         integer                          :: stat
         logical, intent(in)              :: adj, add
         real, dimension(: )              :: x, dat
       end function oper
    end interface
    
    real, dimension( :),intent(in)      :: x
    real, dimension( :),intent(out)     :: g
    real, dimension(size(x))            :: rm,zm
    real, dimension(size(dath))         :: rd,zd
    integer                             :: stat, stat1, i
    real                                :: tmp
    
    
    rd = - dath
    stat = oper(F,T,x,rd)               ! r_d = L x - dat
    
    do i=1,size(rd)   
       tmp = min(1.0,rd(i)/gamdh)
       zd(i) = max(-1.0,tmp)
    end do
    
    stat1 = oper(T,F,g,zd)
    
    if (epsi.ne.0.0) then
       
       if (l1.eq.0) then

          ! Huber
          rm = x       
          do i=1,size(rm)
             tmp = min(1.0,rm(i)/gamxh)  
             zm(i) = max(-1.0,tmp)
          end do
          
       else

          ! Cauchy
          rm = x
          do i=1,size(rm)
             zm(i) = (rm(i)/gamxh**2)/(1 + rm(i)**2/gamxh**2)
          end do

       end if
       
       g = g + epsi*zm
       
    end if
    
  end function hubergdt

  subroutine huber_close()
    deallocate(dath)
  end subroutine huber_close

end module huber_mod

