module huber_solver 

  implicit none
  
contains
  
  subroutine hub(oper, solv, linesearch, fonction, gradient, xc, niter, verb, x0, err) 
    optional                                                               :: x0, err
    interface 
       
       function oper( adj, add, x, dat) result(stat)
         integer             :: stat
         logical, intent(in) :: adj, add
         real, dimension(: ) :: x, dat
       end function oper
       
       function linesearch(oper,fonction,gradient,x,r,a,info) result (stat)
         
         interface
            
            function oper( adj, add, x, dat) result(stat)
              integer                   :: stat
              logical, intent(in)       :: adj, add
              real, dimension(: )       :: x, dat
            end function oper
            
            function fonction(oper,x,fct) result (stat)
              
              interface
                 function oper( adj, add, x, dat) result(stat)
                   integer             :: stat
                   logical, intent(in) :: adj, add
                   real, dimension(: ) :: x, dat
                 end function oper
              end interface
              
              integer             :: stat
              real,dimension(:)   :: x
              real                :: fct
              
            end function fonction
            
            function gradient(oper,x,g) result (stat)
              
              interface
                 function oper( adj, add, x, dat) result(stat)
                   integer             :: stat
                   logical, intent(in) :: adj, add
                   real, dimension(: ) :: x, dat
                 end function oper
              end interface
              
              integer             :: stat
              real,dimension(:)   :: x,g
              
            end function gradient
            
         end interface
         
         integer                       :: stat
         real, dimension(:),intent(in) :: x  ! solution
         real, dimension(:),intent(in) :: r  ! gradient
         real              ,intent(out):: a  ! step
         integer                       :: info            
         
       end function linesearch
       
       function solv(g,x,iter) result (stat)
         
         integer                       :: stat      
         real, dimension (:)           :: g
         real, dimension(:),intent(in) :: x
         integer                       :: iter
         
       end function solv
       
       function fonction(oper,x,fct) result (stat)
         
         interface
            function oper( adj, add, x, dat) result(stat)
	      integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface
         
         integer             :: stat
         real,dimension(:)   :: x
         real                :: fct
         
       end function fonction
       
       function gradient(oper,x,g) result (stat)
         
         interface
            function oper( adj, add, x, dat) result(stat)
	      integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface
         
         integer             :: stat
         real,dimension(:)   :: x,g
         
       end function gradient
       
       
    end interface
    
    
    real, dimension(:),  intent( in)    :: x0                ! initial guess
    integer,             intent( in)    :: niter             ! iterations
    real, dimension(:),  intent(out)    :: xc,err            ! solution and error vector
    real, dimension(size(xc))           :: g                 ! gradient
    integer                             :: iter,stat1,stat2,stat3       
    real                                :: res, f, f0
    real                                :: alpha             ! step after line search
    integer                             :: info, verb
    
    xc=0.
    if (present (x0))  then
       xc=x0
    end if
    
    do iter=0,niter                                 ! Nonlinear Quasi-Newton update       
       
       if (verb>1) write(0,*) 'iteration =',iter,'/',niter
      
       if (present (err)) then
          stat1 = fonction(oper,xc,f)
          if (iter.eq.0) then
             f0 = f
          endif
          err(iter+1) = f/f0
       end if
       
       stat1 = gradient(oper,xc,g)                           ! compute g
       stat2 = solv(g,xc,iter)                               ! compute dx = -Hg
       stat3 = linesearch(oper,fonction,gradient,xc,-g,alpha,info)
       xc = xc - alpha*g                                  ! compute x = x + alp.dx
       
    end do
    
  end subroutine hub
  
end module huber_solver
