module huber_bfgs_b
 
  use cp_lbfgs
  implicit none
  
contains
  
  subroutine lbfgs(oper, fonction, gradient, xc, niter, accuracy, &
  & boundtype, low, up, verb, x0, err) 
    optional               :: x0, err
    interface 

       function oper( adj, add, x, dat) result(stat)
         integer             :: stat
         logical, intent(in) :: adj, add
         real, dimension(: ) :: x, dat
       end function oper

       function fonction(oper,x,fct) result (stat)

         interface
            function oper( adj, add, x, dat) result(stat)
	      integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface

         integer             :: stat
         real,dimension(:)   :: x
         real                :: fct

       end function fonction

       function gradient(oper,x,g) result (stat)

         interface
            function oper( adj, add, x, dat) result(stat)
	      integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface

         integer             :: stat
         real,dimension(:)   :: x,g

       end function gradient


    end interface


    real, dimension(:),  intent( in)    :: x0                ! initial guess
    integer,             intent( in)    :: niter, boundtype  ! iterations
    real, dimension(:),  intent(out)    :: xc,err            ! solution and error vector
    real, dimension(:),  allocatable    :: g                 ! gradient
    integer                             :: iter,stat1,stat2,stat3       
    real                                :: res, f, f0
    real                                :: alpha             ! step after line search
    integer                             :: info, verb

    ! L-BFGS-B variables
    real                            :: dsave(29),factr,pgtol
    real,intent(in)                 :: accuracy
    real,intent(in)                 :: low(:), up(:)
    integer                         :: mmax, nmax
    integer,dimension(:),allocatable:: iwa, nbd
    real,                allocatable:: wa(:)

    character*60                    :: task, csave
    logical                         :: lsave(4), cont
    integer                         :: isave(44), mmem, iprint

    xc=0.
    if (present (x0))  then
       xc=x0
    end if

    nmax = size(xc)
    mmax = 5                                        

    allocate(g(size(xc)))
    allocate(wa(2*mmax*nmax+4*nmax+12*mmax*mmax+12*mmax))
    allocate(iwa(3*nmax))
    allocate(nbd(size(xc)))

    nbd = boundtype
    iprint=0
    mmem=5
    factr=1.0e-16
    pgtol=accuracy
    cont=.true.; task = 'START';iter=0

    stat1 = fonction(oper,xc,f)
    stat2 = gradient(oper,xc,g)

    do while (cont.and.(iter.le.niter))

       cont=.false.

       call setulb(size(xc),mmem,xc,low,up,nbd,f,g,factr,pgtol,wa,iwa,task,iprint,&
       & csave,lsave,isave,dsave)

       if (task(1:2) .eq. 'FG') then

          stat1 = fonction(oper,xc,f)
          stat2 = gradient(oper,xc,g)

          cont=.true.

       end if

       if (task(1:5) .eq. 'NEW_X') then

          cont=.true.
          iter = iter+1

          if (verb.gt.0) then
             write(0,*) 'iter=',iter,'fct=',f
          end if

       end if

    end do

    deallocate(g,nbd,wa,iwa)

  end subroutine lbfgs
  
end module huber_bfgs_b
