module llista_mod
  implicit none
  
  type, private :: entry
     real, dimension (:), pointer  :: g
     type (entry),        pointer  :: next
  end type entry
  
  type list
     private
     type (entry), pointer :: first
     type (entry), pointer :: current
     integer    :: depth
  end type list
  
contains
  
  function llista_init () result (l)
    type (list)  :: l
    
    allocate (l % first)
    l % depth = 1
    call llista_rewind (l)
  end function llista_init
  
  subroutine llista_rewind (l)
    type (list)  :: l
    l % current => l % first
  end subroutine llista_rewind

  function llista_depth (l) result (n)
    type (list)  :: l
    integer      :: n

    n = l % depth
  end function llista_depth

  subroutine llista_add (l, g)
    type (list)                   :: l
    real, dimension (:), pointer  :: g

    l % current % g => g 
    allocate (l % current % next)
    l % depth =  l % depth + 1
    l % current => l % current % next
  end subroutine llista_add

  subroutine llista_down (l, g)
    type (list)                   :: l
    real, dimension (:), pointer  :: g

    g => l % current % g
    l % current => l % current % next
  end subroutine llista_down

  subroutine llista_close (l)
    type (list)  :: l
    integer      :: i

    do i = 1, l % depth
       call llista_chop (l)
    end do

    deallocate (l % first)
  end subroutine llista_close

  subroutine llista_chop (l)
    type (list) :: l

    call llista_rewind (l)
    l % first =>    l % current % next
    if (associated (l % current % g)) &
    deallocate (l % current % g)
    deallocate (l % current)
    l % depth = l % depth - 1
  end subroutine llista_chop   

  subroutine llista_read (n,l,g)
    type (list)                  :: l
    integer                      :: n,i
    real, dimension (:), pointer :: g

    call llista_rewind (l)
    
    if ( (n+1).gt.l%depth) then
       write(0,*) 'size problem'
    end if
     
    do i = 0,n-1
       call llista_down(l,g)
    end do

  end subroutine llista_read

end module llista_mod







