module parainv_under  
  ! parabolic Radon transform in the Fourier domain
  use adj_mod

  implicit none
  integer, private :: nt 
  integer, private :: nx 
  integer, private :: nq 
  integer, private :: nwo
  integer, private :: wght
  integer, private :: alias
  integer, private :: verb
  real, private :: t0 
  real, private :: dt 
  real, private :: q0 
  real, private :: dq 
  real, private :: x0 
  real, private :: dx   
  real, private :: eps
  real, private :: fhi
  real, private :: flo

contains
    
  subroutine parainvunder_init ( t0_in,dt_in,x0_in,dx_in,q0_in,dq_in,nt_in&
  &,nx_in,nq_in,eps_in,weight_in,alias_in,nwo_in,fhi_in,flo_in,verb_in)
    optional:: nwo_in
    real    :: t0_in 
    real    :: dt_in 
    real    :: x0_in 
    real    :: dx_in 
    real    :: q0_in 
    real    :: dq_in 
    real    :: eps_in
    real    :: fhi_in 
    real    :: flo_in

    integer    :: nt_in 
    integer    :: nx_in 
    integer    :: nq_in
    integer    :: nwo_in
    integer    :: weight_in
    integer    :: alias_in
    integer    :: verb_in
    
    t0 = t0_in 
    dt = dt_in 
    x0 = x0_in 
    dx = dx_in 
    q0 = q0_in 
    dq = dq_in 
    nt = nt_in 
    nx = nx_in 
    nq = nq_in
    nwo= nwo_in
    eps = eps_in
    wght=weight_in
    alias=alias_in
    verb=verb_in
    fhi = fhi_in
    flo = flo_in

  end subroutine parainvunder_init
  
  function parainvunder_lop ( adj, add, modl, data, wout) result(stat)
    optional::wout
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: modl,data
    real,dimension(:),intent(out):: wout
    call adjnull (adj,add,modl,data)
    call parainvunder_lop2(adj,modl,data,wout)
    stat=0
  end function parainvunder_lop
  
  subroutine parainvunder_lop2(adj,modl,data,wout)
    optional::wout
    logical,intent(in) :: adj 
    real, dimension (nt,nq)  :: modl 
    real, dimension (nt,nx)  :: data
    real, dimension (nwo,nq)  :: wout
    complex*16, allocatable, dimension(:,:) :: cdata,cmodl
    complex*16, allocatable, dimension(:,:) :: mat,tmp,l,rhs
    complex*16, allocatable, dimension(:)   :: res
    complex*16, allocatable, dimension(:)   :: work
    real,       allocatable, dimension(:,:) :: weight
    integer,dimension(nx) :: ipiv
    integer    :: iz,ix,iq,it,iw,stat,iq2,nw,info,lwork,nwmax,nwmin
    real       :: x,q,w,sx,t,z,wt,w0,dw,pi,q2,taper,wtap,wmin,thresh,a
    complex*16 :: cc
    
    pi=3.14159265
    lwork=10000
    nw = jpad2(nt) 
    w0 = -pi/dt   ;  dw= 2.*pi/(nw*dt)  

    nwmax=ceiling(fhi*nw*dt+nw/2+0.5)
    nwmin=ceiling(flo*nw*dt+nw/2+0.5)
    allocate(work(lwork))
    allocate(cdata(nw,nx),cmodl(nw,nq))
    allocate(tmp(nq,nx),l(nx,nq))
    allocate(rhs(nq,nx),res(nq),mat(nx,nx),weight(nq,nq))
    cmodl=cmplx(0.,0.)
    cdata=cmplx(0.,0.)
    
    ! Fourier transform the data

    if (adj) then
       
       do ix=1,nx       
          cdata(1:nt,ix)=cmplx(data(:,ix),0.)
       end do

       call dft1axis(0,1.,nw,nx,cdata)

    else
       
       do iq=1,nq       
          cmodl(1:nt,iq)=cmplx(modl(:,iq),0.)
       end do
       
       call dft1axis(0,1.,nw,nq,cmodl)

       
    end if
   
    ! End Fourier transform

    weight=0.
    
    do iq=1,nq
       weight(iq,iq)=1.
    end do
  
    do iw= nwmin,nwmax
       
       w = w0 + (iw-1)*dw
       a = iw
       
       if (verb>0) write(0,*) 'PRT>> frequency=',w/(2.*pi),'/',fhi

         l=cmplx(0.,0.)
       rhs=cmplx(0.,0.)
       mat=cmplx(0.,0.)
       res=cmplx(0.,0.)
       
       do iq =1 ,nq
          q = q0 + (iq-1) * dq

          do ix= 1, nx  
             x = x0 + (ix-1) * dx

             if (alias.ne.0) then

                wmin = pi/(2*alias*abs(dx*q*x))
                wtap = 7*wmin/8
                wmin = 9*wmin/8

                if (w.lt.wtap) then
                   taper=1.
                else if (w.lt.wmin) then
                   taper=cos(pi*(w-wtap)/(2*(wmin-wtap)))
                else
                   taper=0.
                end if

             else

                taper=1.

             end if

             if (adj) then
                
                l(ix,iq) = cexp(cmplx(0.,w*x*x*q))*taper

             else
                
                cc = cexp(cmplx(0.,w*x*x*q))
                cdata(iw,ix)= cdata(iw,ix) + 2*cmodl(iw,iq)*cc*taper
                
             end if
             
          end do
          
       end do
       
       if (adj) then
          
          rhs=matmul(weight,transpose(conjg(l))) ! rhs = WL'   (nq x nx)
          mat=matmul(l,rhs) 

          do ix=1,nx
!             mat(ix,ix)=mat(ix,ix)+log(a+1)*eps  ! mat = LWL'+ epsI   (nx x nx)
             mat(ix,ix)=mat(ix,ix)+eps  ! mat = LWL'+ epsI   (nx x nx)
          end do
          call zgetrf(nx,nx,mat,nx,ipiv,info)
          call zgetri(nx,mat,nx,ipiv,work,lwork,info)  ! Inversion
          tmp=matmul(rhs,mat)                    ! tmp = WL'(LWL'+ epsI)-1 (nq x nx)
          res=matmul(tmp,cdata(iw,:))            ! res = WL'(LWL'+ epsI)-1 data(iw,:) (nq)

          cmodl(iw,:)=2*res                      ! Half of the frequencies

          if (present(wout)) then
             if (wght.eq.1) then
                wout(iw,:)=log(abs(res)+1)/maxval(log(abs(res)+1))
             else
                wout(iw,:)=(abs(res))/maxval(abs(res))
             end if
          end if

          if (wght.ne.0) then
             do iq=1,nq
                if (wght.eq.1) then
                   weight(iq,iq)=log(abs(res(iq))+1)/maxval(log(abs(res)+1))
                else
                   weight(iq,iq)=(abs(res(iq))/maxval(abs(res)))
                end if
             end do
          end if

       end if
       
    end do

    if (adj) then
       
       call dft1axis(1,1.,nw,nq,cmodl)
       
       do iq=1,nq
          modl(:,iq)=real(cmodl(1:nt,iq))
       end do
       
    else
       
       call dft1axis(1,1.,nw,nx,cdata)
       
       do ix=1,nx 
          data(:,ix)=real(cdata(1:nt,ix))
       end do
       
    end if  
    
    deallocate(work,cdata,cmodl)
    deallocate(tmp,l,rhs,res,mat,weight)

  end subroutine parainvunder_lop2
  
  integer function jpad2(n)
    integer n
    jpad2=1
    do
       if (jpad2.ge.n) exit
       jpad2=jpad2*2
    end do
  end function jpad2
  
end module parainv_under





