program CAL3
  use sep
  use ftu_mod
  use filter_mod

  implicit none

  integer 				:: n1,n2,n3,n2s,n3s
  integer 				:: n1_aux,n2_aux,n3_aux
  real					:: d1,d2,d3
  real 					:: d1_aux,d2_aux,d3_aux
  real 					:: o1,o2,o3
  real 					:: o1_aux,o2_aux,o3_aux
  integer				:: i1, i2, i3, iver, test, ioff, w1, wb, we
  integer				:: flength, iw
  logical                               :: auxi,ver
  real					:: amp, eps, wd, wv, sign, Io
  real					:: deltat, stime, w, pi, w0, dw
  complex				:: Z, Zo
  real, dimension(:), allocatable	:: filt, filt2, filtmp
  real, dimension(:,:), allocatable	:: pdata, zdata
  complex, dimension(:), allocatable	:: pcmpx, zcmpx

  call sep_init()

! ----- calling parameters -------

  call from_history ("n1", n1)
  call from_history ("d1", d1)
  call from_history ("o1", o1)

  call from_history ("n2", n2)
  call from_history ("d2", d2)
  call from_history ("o2", o2)

  call from_history ("n3", n3, 1)
  call from_history ("d3", d3, 1.0)
  call from_history ("o3", o3, 0.0)

  call from_aux	("zdata","n1", n1_aux)
  call from_aux ("zdata","n2", n2_aux)
  call from_aux ("zdata","n3", n3_aux, 1)

  call from_aux ("zdata","d1", d1_aux)
  call from_aux ("zdata","d2", d2_aux)
  call from_aux ("zdata","d3", d3_aux, 1.0)

  call from_aux ("zdata","o1", o1_aux)
  call from_aux ("zdata","o2", o2_aux)
  call from_aux ("zdata","o3", o3_aux, 0.0)
 
  call from_param( "verb", iver, 1); ver = (iver==1)
  call from_param( "eps", eps, 0.00001)
  call from_param( "wd", wd, 118.0)
  call from_param( "wv", wv, 1500.0) 
  call from_param( "stime", stime, 0.2)
  call from_param( "flength", flength, 16)
  call from_param( "w1", w1, 10) 

  call to_history ("o1", o1 )
  call to_history ("d1", d1 )
  call to_history ("n1", flength)

  call to_history ("o2", o3 )
  call to_history ("d2", d3 )
  call to_history ("n2", n3 )

  call to_history("verb", iver)

  auxi = exist_file ("zdata")

  call sep_close ()
! ----- end calling parameters -------

! ----- checking P and Z are consistent -------

  if (.not. auxi) call seperr ("Need Z data file")

  if (n1 /= n1_aux) call seperr ("n1 of P is different than n1 of Z")
  if (n2 /= n2_aux) call seperr ("n2 of P is different than n2 of Z")
  if (n3 /= n3_aux) call seperr ("n3 of P is different than n3 of Z")

  if (o1 /= o1_aux) call seperr ("o1 of P is different than o1 of Z")
  if (o2 /= o2_aux) call seperr ("o2 of P is different than o2 of Z")
  if (o3 /= o3_aux) call seperr ("o3 of P is different than o3 of Z")

  if (d1 /= d1_aux) call seperr ("d1 of P is different than d1 of Z")
  if (d2 /= d2_aux) call seperr ("d2 of P is different than d2 of Z")
  if (d3 /= d3_aux) call seperr ("d3 of P is different than d3 of Z")

! ----- finishing consistency -------------

  if (ver) write(0,*) "files are okay"

! ----- allocating -------------

   
  if (allocated (pdata) ) deallocate (pdata)
  if (allocated (zdata) ) deallocate (zdata)
  if (allocated (pcmpx) ) deallocate (pcmpx)
  if (allocated (zcmpx) ) deallocate (zcmpx)
  if (allocated (filt) ) deallocate (filt)
  if (allocated (filt2) ) deallocate (filt2)
  if (allocated (filtmp) ) deallocate (filtmp)
   
if (ver) write(0,*) "Reserving space ..."

  allocate (pdata(n1,n2), STAT=test)
	if(test /=0) call seperr("cant allocate pdata")
  allocate (zdata(n1,n2), STAT=test)
	if(test /=0) call seperr("cant allocate zdata")
  allocate (pcmpx(n1),  STAT=test)
	if(test /=0) call seperr("cant allocate pcmpx")
  allocate (zcmpx(n1),  STAT=test)
	if(test /=0) call seperr("cant allocate zcmpx")
  allocate (filt(flength),  STAT=test)
	if(test /=0) call seperr("cant allocate filt")
  allocate (filt2(flength),  STAT=test)
	if(test /=0) call seperr("cant allocate filt2")
  allocate (filtmp(flength),  STAT=test)
	if(test /=0) call seperr("cant allocate filtmp")

if (ver) write(0,*) "Done reserving space ..."

Io=1. ! Io normalized to 1 - not sure about this

pi=3.14159265

sign=+1.0

filt2=0
n2s=0
n3s=0

do i3=1,n3 !{

call sep_read(pdata)
call sep_read(zdata,"zdata")

filtmp=0

 do i2=1,n2 !{

  pcmpx = cmplx(pdata(:,i2))
  zcmpx = cmplx(zdata(:,i2))

  call ftu(sign,n1,pcmpx)
  call ftu(sign,n1,zcmpx)

  w0=-pi/d1; dw=2.*pi/(n1*d1)
  Zo=cmplx(1.0)

   do iw=1,n1 !{

     w = w0+(iw-1)*dw
     deltat=2*wd/wv
     Z=cexp(cmplx(0.0,w*deltat))

     pcmpx(iw) = (Zo+Z)*(0.5)*pcmpx(iw)
     zcmpx(iw) = (Zo-Z)*Io*(0.5)*zcmpx(iw)

   end do !}

  call ftu(-sign,n1,pcmpx)
  call ftu(-sign,n1,zcmpx)

  pdata(:,i2)=real(pcmpx)
  zdata(:,i2)=real(zcmpx)

  if (ver) write(0,*) "Done calibrating"

  call filter(pdata(:,i2),zdata(:,i2),n1,o1,d1,eps,stime,flength,filt) 

  if (sum(filt) /= 0.0) n2s=n2s+1

  filtmp(:)=filt(:)+filtmp(:)

 end do !}

 filt2(:)=filt2(:)+filtmp(:)

end do !}

if (ver) write(0,*) "Done i3,n2s,n3", i3, n2s,n3 
filt2=filt2/(n2s+n3)

call sep_write(filt2)

deallocate( pdata, STAT=test)
if (test /=0) call seperr("cant deallocate pdata")
deallocate( zdata, STAT=test)
if (test /=0) call seperr("cant deallocate zdata")
deallocate( pcmpx, STAT=test)
if (test /=0) call seperr("cant deallocate pcmpx")
deallocate( zcmpx, STAT=test)
if (test /=0) call seperr("cant deallocate zcmpx")
deallocate( filt, STAT=test)
if (test /=0) call seperr("cant deallocate filt")
deallocate( filt2, STAT=test)
if (test /=0) call seperr("cant deallocate filt2")
deallocate( filtmp, STAT=test)
if (test /=0) call seperr("cant deallocate filtmp")

if (ver) write(0,*) "all done"

  call exit(0)
end program CAL3
