program FKfilt
!-----------------------------------------------------------------
! Fourier transform and apply dip filter
!-----------------------------------------------------------------
	use sep

	implicit none

	integer                               :: nw, nk, iw, ik
	integer                               :: imute, iwrite_fk
	real                                  :: dw, dk, d1, d2, w, k, w0, k0
	real                                  :: vel, wt, velmin, velmax
	real                                  :: taper_range, taper
	complex, allocatable, dimension (:,:) :: data
	logical                               :: mute, write_fk

	call sep_init()
	call from_history( "n1", nw)
	call from_history( "n2", nk)
	call from_either( "d1", d1)
	call from_either( "d2", d2)
	call from_par( "write_fk", iwrite_fk, 0);   write_fk = (iwrite_fk==1)
	call from_par( "mute", imute);              mute = (imute==1)
	call from_par( "velmin", velmin, 0.0)
	call from_par( "velmax", velmax, 1.0e15)

	call to_history( "n1", nw)
	call to_history( "n2", nk)
	if( write_fk)   call to_history( "n3", 2)
	call to_history( "velmin", velmin)
	call to_history( "velmax", velmax)
	call to_history( "d1", d1)
	call to_history( "d2", d2)

	allocate( data(nw,nk) )
	call sep_read( data)

	dw =  1/((nw-1)*d1); dk =  1/((nk-1)*d2)
	w0 = -1/(2*d1)     ; k0 = -1/(2*d2)
	taper_range = 0.1

	call ft1axis(0, -1., nw, nk, data)
	call ft2axis(0, 1., nw, nk, data)
	
	
	if( write_fk .and. .not. mute) then
!		call sep_write( cabs(data))        ! Write F-K data
		call sep_write( real(data))        ! Write F-K data
		call sep_write( cmplx(data))
	end if

	if( mute) then
		do iw = 1,nw  
			w = (iw - 1) * dw + w0
			do ik = 1,nk  
				k = (ik - 1) * dk  + k0
				if( k /= 0.0) then
					vel = w/k
					if( vel .ge. velmin .and. vel .le. velmax) then
						wt = 1.0
						if( vel .gt. (1-taper_range)*velmax .and. vel .le. velmax) then
							taper = (1 - vel/velmax)/taper_range
						else if( vel .lt. (1-taper_range)*velmin .and. vel .ge. velmin) then
							taper = (1 - vel/velmin)/taper_range
						else
							taper = 1.0
						end if 
					else
						wt = 0.0
					end if 
				else
					wt = 0.0
				end if 
				data(iw,ik) = data(iw,ik)*wt*taper
			end do 
		end do 
	
		if( write_fk) then
!			call sep_write( cabs(data) )
			call sep_write( real(data) )
			call sep_write( cmplx(data) )
		end if

	end if

	call ft1axis(1, -1., nw, nk, data)
	call ft2axis(1, 1., nw, nk, data)

	if( .not. write_fk)  call sep_write( data)           ! Write T-X data

end program FKfilt
