program PaZ
  use sep

  implicit none

  integer 				:: n1,n2,n3
  integer 				:: n1_aux,n2_aux,n3_aux
  real					:: d1,d2,d3
  real 					:: d1_aux,d2_aux,d3_aux
  real 					:: o1,o2,o3
  real 					:: o1_aux,o2_aux,o3_aux
  integer				:: i1, i2, i3, iver, test, w1, flength
  integer				:: iloc, b1, e1, sloc
  logical                               :: auxi,ver
  real					:: num, den, alpha, eps
  real					:: stime
  real, dimension(:), allocatable	:: pamp, zamp
  real, dimension(:,:), allocatable	:: pdata, zdata, pzres

  call sep_init()

! ----- calling parameters -------

  call from_history ("n1", n1)
  call from_history ("d1", d1)
  call from_history ("o1", o1)

  call from_history ("n2", n2)
  call from_history ("d2", d2)
  call from_history ("o2", o2)

  call from_history ("n3", n3, 1)
  call from_history ("d3", d3, 1.0)
  call from_history ("o3", o3, 0.0)

  call from_aux	("zdata","n1", n1_aux)
  call from_aux ("zdata","n2", n2_aux)
  call from_aux ("zdata","n3", n3_aux, 1)

  call from_aux ("zdata","d1", d1_aux)
  call from_aux ("zdata","d2", d2_aux)
  call from_aux ("zdata","d3", d3_aux, 1.0)

  call from_aux ("zdata","o1", o1_aux)
  call from_aux ("zdata","o2", o2_aux)
  call from_aux ("zdata","o3", o3_aux, 0.0)
 
  call from_param( "verb", iver, 1); ver = (iver==1)
  call from_param( "eps", eps, 0.0001)
  call from_param( "w1", w1, 375)  
  call from_param( "stime", stime, 0.2)
  call from_param( "flength", flength, 16)

  call to_history ("o1", o1)
  call to_history ("d1", d1)
  call to_history ("n1", n1)

  call to_history ("o2", o2)
  call to_history ("d2", d2)
  call to_history ("n2", n2)

  call to_history ("o3", o3)
  call to_history ("d3", d3)
  call to_history ("n3", n3)

  call to_history("verb", iver)

  auxi = exist_file ("zdata")

  call sep_close ()
! ----- end calling parameters -------

! ----- checking P and Z are consistent -------

  if (.not. auxi) call seperr ("Need Z data file")

  if (n1 /= n1_aux) call seperr ("n1 of P is different than n1 of Z")
  if (n2 /= n2_aux) call seperr ("n2 of P is different than n2 of Z")
  if (n3 /= n3_aux) call seperr ("n3 of P is different than n3 of Z")

  if (o1 /= o1_aux) call seperr ("o1 of P is different than o1 of Z")
  if (o2 /= o2_aux) call seperr ("o2 of P is different than o2 of Z")
  if (o3 /= o3_aux) call seperr ("o3 of P is different than o3 of Z")

  if (d1 /= d1_aux) call seperr ("d1 of P is different than d1 of Z")
  if (d2 /= d2_aux) call seperr ("d2 of P is different than d2 of Z")
  if (d3 /= d3_aux) call seperr ("d3 of P is different than d3 of Z")

! ----- finishing consistency -------------

  if (ver) write(0,*) "files are okay"

! ----- allocating -------------

   
  if (allocated (pdata) ) deallocate (pdata)
  if (allocated (zdata) ) deallocate (zdata)
  if (allocated (pzres) ) deallocate (pzres)
  if (allocated (pamp) ) deallocate (pamp)
  if (allocated (zamp) ) deallocate (zamp)
   
  allocate (pdata(n1,n2), STAT=test)
	if(test /=0) call seperr("cant allocate pdata")
  allocate (zdata(n1,n2), STAT=test)
	if(test /=0) call seperr("cant allocate zdata")
  allocate (pzres(n1,n2),  STAT=test)
	if(test /=0) call seperr("cant allocate pzres")
  allocate (pamp(flength),  STAT=test)
	if(test /=0) call seperr("cant allocate pamp")
  allocate (zamp(flength),  STAT=test)
	if(test /=0) call seperr("cant allocate zamp")

if (ver) write(0,*) "Done allocating"

do i3=1,n3
call sep_read (pdata)
call sep_read (zdata,"zdata")
 do i2=1,n2
  if (ver) write(0,*) "trace", i2
  iloc = maxloc(abs(pdata(:,i2)),1)
  sloc = 1.5+(stime-o1)/d1
  b1 = sloc+iloc
  e1 = sloc+iloc+flength
  pamp(:) = pdata(b1:e1,i2)
  zamp(:) = zdata(b1:e1,i2)

  num = dot_product(pamp,zamp)
  den = dot_product(zamp,zamp)

  alpha = (num)/(den + eps)
  if (ver) write(0,*) "num,den,alpha", num,den,alpha
  pzres(:,i2) = pdata(:,i2) + alpha*zdata(:,i2)
 end do
call sep_write (pzres)
end do

deallocate( pdata, STAT=test)
if (test /=0) call seperr("cant deallocate pdata")
deallocate( zdata, STAT=test)
if (test /=0) call seperr("cant deallocate zdata")
deallocate( pzres, STAT=test)
if (test /=0) call seperr("cant deallocate pzres")
deallocate( zamp, STAT=test)
if (test /=0) call seperr("cant deallocate zamp")
deallocate( pamp, STAT=test)
if (test /=0) call seperr("cant deallocate pamp")

if (ver) write(0,*) "all done"

call exit(0)
end program PaZ
