!-------------------------------------------------------------------------------
program RadialTrace

	use radial_trace
	use sep
	use solver_mod
	use cgstep_mod

	implicit none

!-------------------------------------------------------------------------------
	integer :: nt, nx1, nx2, nv1, nv2, ny1, ny2
	integer :: ivint, ilint, niter, stat, iverb, iadj, iinv, inorm, iy1, iy2
	real    :: dt, dx1, dx2, dy1, dy2, dv1, dv2, t0, o1, x01, x02, y01, y02,     &
	           v01, v02, vmax1, vmax2

	logical :: adj, inv, norm, lint, vinterp, verb

	real, dimension(:), pointer :: radial, timespace

	call sep_init()

	call from_param( "adj",     iadj,  1);  adj  = (iadj==1)
	call from_param( "inv",     iinv,  0);  inv  = (iinv==1)
	call from_param( "norm",    inorm, 1);  norm = (inorm==1)
	call from_param( "vinterp", ivint, 1);  vinterp = (ivint==1) 
	call from_param( "lint",    ilint, 1);  lint = (ilint==1)
	call from_param( "verb",    iverb, 0);  verb = (iverb==1)

	call from_history( "n1", nt)	
	call from_history( "d1", dt)	
	call from_history( "o1", o1)
	call from_param(  "t0", t0, o1)      ! Allow for user definition of t0.
	call from_history( "n4", ny1, 1  );  call from_history( "n5", ny2, 1  )
	call from_history( "d4", dy1, 1.0);  call from_history( "d5", dy2, 1.0)
	call from_history( "o4", y01, 0.0);  call from_history( "o5", y02, 0.0)

	if( adj) then
		call from_history( "n2", nx1);  call from_history( "n3", nx2, 1  )
		call from_history( "d2", dx1);  call from_history( "d3", dx2, 1.0)
		call from_history( "o2", x01);  call from_history( "o3", x02, 0.0)
		call from_param( "nv1", nv1);   call from_param( "nv2", nv2)
		call from_param( "vmin1", v01)
		call from_param( "vmax1", vmax1)
		call from_param( "vmin2", v02)
		call from_param( "vmax2", vmax2)
		
		dv1 = (vmax1-v01)/(nv1-1);  dv2 = (vmax2-v02)/(nv2-1)

		call to_history( "n2", nv1);  call to_history( "n3", nv2)
		call to_history( "d2", dv1);  call to_history( "d3", dv2)
		call to_history( "o2", v01);  call to_history( "o3", v02)
	else
		call from_history( "n2", nv1);  call from_history( "n3", nv2)
		call from_history( "d2", dv1);  call from_history( "d3", dv2)
		call from_history( "o2", v01);  call from_history( "o3", v02)
		call from_param( "nx1", nx1);   call from_param( "nx2", nx2, 1  )
		call from_param( "dx1", dx1);   call from_param( "dx2", dx2, 1.0)
		call from_param( "x01", x01);   call from_param( "x02", x02, 0.0)
		call to_history( "n2", nx1);  call to_history( "n3", nx2)
		call to_history( "d2", dx1);  call to_history( "d3", dx2)
		call to_history( "o2", x01);  call to_history( "o3", x02)
	end if
	call to_history( "n1", nt)
	call to_history( "d1", dt)
	call to_history( "o1", o1)
	call to_history( "n4", ny1);  call to_history( "n5", ny2)
	call to_history( "d4", dy1);  call to_history( "d5", dy2)
	call to_history( "o4", y01);  call to_history( "o5", y02)

	call sep_close()

!-------------------------------------------------------------------------------
	allocate( radial(nt*nv1*nv2), timespace(nt*nx1*nx2) )

	call radial_trace_init( nt, nx1, nx2, nv1, nv2, dt, dx1, dx2, dv1, dv2, t0,  &
	                        x01, x02, v01, v02, vinterp, lint, norm, verb=verb   )

	do iy2=1,ny2
		do iy1=1,ny1

			write(0,*) 'shot',iy1,iy2,' of',ny1,ny2

			if( adj) then
				call sep_read( timespace)
			else
				call sep_read( radial)
			end if

			stat = radial_trace_exec( adj, .false., radial, timespace)

			if( adj) then
				call sep_write( radial)
			else
				call sep_write( timespace )
			end if

		end do
	end do
	
	deallocate( timespace, radial)
	call radial_trace_close()

	call exit(0)

end program RadialTrace
!-------------------------------------------------------------------------------
