!!$=head1 NAME
!!$
!!$Vrms2int - convert RMS velocity to interval velocity
!!$
!!$=head1 SYNOPSIS
!!$
!!$Vrms2int < vrms.H weight= vrms= niter= eps= > vint.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Least-square inversion, preconditioned by integration.
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from history
!!$
!!$B<integer> I<n1, n2>
!!$
!!$=item from par
!!$
!!$B<integer> I<niter> - number of iterations
!!$
!!$B<real> I<eps> - scaling for preconditioning
!!$
!!$B<filename> I<weight> - data weight for inversion 
!!$
!!$B<filename> I<vrms> - predicted RMS velocity
!!$
!!$=back
!!$
!!$=head1 EXAMPLES
!!$
!!$F<gee/prc/bob/>
!!$
!!$=head1 MODULES
!!$
!!$F<vrms2int.r90>
!!$
!!$=cut
!%
program rms2int
  use sep
  use vrms2int_mod

  implicit none
  integer                          :: i2, n1, n2, niter
  real                             :: eps
  real, dimension (:), allocatable :: vint, vrms
  real, dimension (:), pointer     :: mywt

  call sep_init (SOURCE)
  call from_history (n1)
  call from_either ("n2",n2,1)

  call from_par ("niter",niter,20)
  call from_par ("eps",eps,1.)

  call to_history ("title","Interval velocity")
  
  call to_history ("n1",n1,"vrms")
  call to_history ("n2",n2,"vrms")
  call to_history ("title","RMS velocity","vrms")
  call to_history ("esize",4,"vrms")
  call sep_close ()

  allocate (vint (n1), vrms (n1), mywt (n1))
  do i2= 1, n2 
     call sep_read (vrms)
     call sep_read (mywt, file="weight")
     call vrms2int (niter, eps, mywt, vrms, vint)
     call sep_write (vint) 
     call sep_write (vrms, "vrms")
  end do

  deallocate (vint, vrms, mywt)
  call exit (0)
end program rms2int

