module morgan_util

use sep

implicit none

contains

! ---------------------------------------------------------------------------
subroutine oper_stats( label, top, model, data, adj, ithr)
	real,    dimension(:), intent(IN) :: model, data
	logical,               intent(IN) :: top, adj
	character(len=*),      intent(IN) :: label
	integer                           :: ithr
	optional                          :: ithr

	if( .NOT. present(ithr) ) ithr=0

	if( top) then
		if( adj) then
			write(0,"(a,a,i2,a,3f)") trim(label),"TOP: adj=T, ithr=",ithr, &
			                          "  Data min/max/avg:",minval(data),maxval(data),sum(data)/size(data)
		else
			write(0,"(a,a,i2,a,3f)") trim(label),"TOP: adj=F, ithr=",ithr, &
			                          " Model min/max/avg:",minval(model),maxval(model),sum(model)/size(model)
		end if
	else
		if( adj) then
			write(0,"(a,a,i2,a,3f)") trim(label),"BOT: adj=T, ithr=",ithr, &
			                          " Model min/max/avg:",minval(model),maxval(model),sum(model)/size(model)
		else
			write(0,"(a,a,i2,a,3f)") trim(label),"BOT: adj=F, ithr=",ithr, &
			                          "  Data min/max/avg:",minval(data),maxval(data),sum(data)/size(data)
		end if
	end if

end subroutine oper_stats
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
integer function grid2line2( nn, ii) result(i)
  integer, intent(IN)  :: nn(2), ii(2)

	i = (ii(2)-1)*nn(1) + ii(1)
end function grid2line2
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
integer function grid2line3( nn, ii) result(i)
  integer, intent(IN)  :: nn(3), ii(3)
	integer              :: i2

	i2 = grid2line2( nn(1:2), ii(1:2) )
	i = (ii(3)-1)*product(nn(1:2)) + i2
end function grid2line3
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
integer function grid2line4( nn, ii) result(i)
  integer, intent(IN)  :: nn(4), ii(4)
	integer              :: i3

	i3 = grid2line3( nn(1:3), ii(1:3) )
	i = (ii(4)-1)*product(nn(1:3)) + i3
end function grid2line4
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
integer function grid2line5( nn, ii) result(i)
  integer, intent(IN)  :: nn(5), ii(5)
	integer              :: i4

	i4 = grid2line4( nn(1:4), ii(1:4) )
	i = (ii(5)-1)*product(nn(1:4)) + i4
end function grid2line5
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
integer function grid2line6( nn, ii) result(i)
  integer, intent(IN)  :: nn(6), ii(6)
	integer              :: i5

	i5 = grid2line5( nn(1:5), ii(1:5) )
	i = (ii(6)-1)*product(nn(1:5)) + i5
end function grid2line6
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
subroutine fromaux_batch( tag, n1, d1, o1, n2, d2, o2, n3, d3, o3, n4, d4,  &
                          o4, n5, d5, o5, n6, d6, o6, n7, d7, o7 )
	character(len=*), intent(IN)  :: tag
	integer,          intent(OUT) :: n1, n2
	real,             intent(OUT) :: d1, d2, o1, o2
	integer                       :: n3, n4, n5, n6, n7
	real                          :: d3, d4, d5, d6, d7, o3, o4, o5, o6, o7
	optional                      :: n3, d3, o3, n4, d4, o4, n5, o5, d5, n6,  &
	                                 o6, d6, n7, o7, d7

	call from_aux(tag,'n1',n1);  call from_aux(tag,'n2',n2)
	call from_aux(tag,'d1',d1);  call from_aux(tag,'d2',d2)
	call from_aux(tag,'o1',o1);  call from_aux(tag,'o2',o2)

	if(present(n3)) call from_aux(tag,'n3',n3)
	if(present(d3)) call from_aux(tag,'d3',d3)
	if(present(o3)) call from_aux(tag,'o3',o3)
	if(present(n4)) call from_aux(tag,'n4',n4)
	if(present(d4)) call from_aux(tag,'d4',d4)
	if(present(o4)) call from_aux(tag,'o4',o4)
	if(present(n5)) call from_aux(tag,'n5',n5)
	if(present(d5)) call from_aux(tag,'d5',d5)
	if(present(o5)) call from_aux(tag,'o5',o5)
	if(present(n6)) call from_aux(tag,'n6',n6)
	if(present(d6)) call from_aux(tag,'d6',d6)
	if(present(o6)) call from_aux(tag,'o6',o6)
	if(present(n7)) call from_aux(tag,'n7',n7)
	if(present(d7)) call from_aux(tag,'d7',d7)
	if(present(o7)) call from_aux(tag,'o7',o7)
end subroutine fromaux_batch
! ---------------------------------------------------------------------------

! ---------------------------------------------------------------------------
subroutine tohistory_batch( tag, n1, d1, o1, n2, d2, o2, n3, d3, o3, n4,    &
                            d4, o4, n5, d5, o5, n6, d6, o6, n7, d7, o7 )
	character(len=*), intent(IN)  :: tag
	integer,          intent(OUT) :: n1, n2
	real,             intent(OUT) :: d1, d2, o1, o2
	integer                       :: n3, n4, n5, n6, n7
	real                          :: d3, d4, d5, d6, d7, o3, o4, o5, o6, o7
	optional                      :: n3, d3, o3, n4, d4, o4, n5, o5, d5, n6,  &
	                                 o6, d6, n7, o7, d7

	call to_history('n1',n1,tag);  call to_history('n2',n2,tag)
	call to_history('d1',d1,tag);  call to_history('d2',d2,tag)
	call to_history('o1',o1,tag);  call to_history('o2',o2,tag)

	if(present(n3)) call to_history('n3',n3,tag)
	if(present(d3)) call to_history('d3',d3,tag)
	if(present(o3)) call to_history('o3',o3,tag)
	if(present(n4)) call to_history('n4',n4,tag)
	if(present(d4)) call to_history('d4',d4,tag)
	if(present(o4)) call to_history('o4',o4,tag)
	if(present(n5)) call to_history('n5',n5,tag)
	if(present(d5)) call to_history('d5',d5,tag)
	if(present(o5)) call to_history('o5',o5,tag)
	if(present(n6)) call to_history('n6',n6,tag)
	if(present(d6)) call to_history('d6',d6,tag)
	if(present(o6)) call to_history('o6',o6,tag)
	if(present(n7)) call to_history('n7',n7,tag)
	if(present(d7)) call to_history('d7',d7,tag)
	if(present(o7)) call to_history('o7',o7,tag)
end subroutine tohistory_batch
! ---------------------------------------------------------------------------

end module morgan_util
