module radial_trace_mod

	integer, save, private        :: nt, nx, nv, method
	real, save, private           :: dt, t0, dx, x0, vmin, vmax, dv

	contains

!---------------------------------------------------------------------------
	subroutine radial_trace_init( nti, nxi, nvi, dti, dxi, t0i, x0i, &
                                vmini, vmaxi, methodi )
		integer                     :: nti, nxi, nvi
		real                        :: dti, t0i, dxi, x0i, vmini, vmaxi

		nt=nti;  nx=nxi;  nv=nvi
		dt=dti;  dx=dxi
		t0=t0i;  x0=x0i
		vmin=vmini;  vmax=vmaxi
		method = methodi

		dv = (vmax-vmin)/nv
	end subroutine radial_trace_init
!---------------------------------------------------------------------------
	integer function radial_trace_exec( adj, add, timespace_io, radial_io )
		real, dimension(:), pointer    :: timespace_io, radial_io
		real, dimension(:,:), pointer  :: timespace, radial
		logical                        :: adj, add

		allocate( timespace(nt,nx), radial(nt,nv) )

		timespace = reshape( timespace_io, (/nt,nx/) )
		radial    = reshape( radial_io,    (/nt,nv/) )

		if( adj) then
			timespace = 0
		else
			radial = 0
		end if

		if( method == 1) then
write(0,*) 'DOING V-interp'
			call radial_trace_vinterp( adj, timespace, radial)
		else if( method == 2) then
write(0,*) 'DOING T-interp'
			call radial_trace_tinterp( adj, timespace, radial)
		else
write(0,*) 'DOING X-interp'
			call radial_trace_xinterp( adj, timespace, radial)
		end if

		timespace_io = reshape( timespace, (/nt*nx/) )
		radial_io    = reshape( radial,    (/nt*nv/) )

		radial_trace_exec = 0

	end function radial_trace_exec
!---------------------------------------------------------------------------
	subroutine radial_trace_vinterp( adj, timespace, radial )
		integer                       :: it, ix, iv0, iv1
		real                          :: t, x, v, c0, c1
		real, dimension(:,:), pointer :: timespace, radial
		logical                       :: adj
		
		do it = 1,nt
			t = (it-1)*dt + t0
			do ix = 1,nx
				x = (ix-1)*dx + x0
				v = x/t
				iv0 = floor((v-vmin)/dv);  iv1 = iv0+1 
				if( iv1 <= nv .and. iv0 > 0 ) then
					c1 = (v-vmin)/dv - iv0;  c0 = 1.0 - c1
					if( adj) then
						timespace( it, ix) = c0*radial( it,iv0) + c1*radial( it,iv1)
					else
						radial( it, iv0) = radial( it, iv0) + c0*timespace( it, ix)
						radial( it, iv1) = radial( it, iv1) + c1*timespace( it, ix)
					end if
				end if
			end do
		end do
	end subroutine radial_trace_vinterp
!---------------------------------------------------------------------------
	subroutine radial_trace_tinterp( adj, timespace, radial )
		integer                       :: iv, ix, it
		real                          :: t, x, v
		real, dimension(:,:), pointer :: timespace, radial
		logical                       :: adj
		
		do ix = 1,nx
			x = (ix-1)*dx + x0
			do iv = 1,nv
				v = (iv-1)*dv + vmin
				t = x/v
				it = nint((t-t0)/dt)
				if( it <= nt .and. it > 0 ) then
					if( adj) then
						timespace( it, ix) = timespace( it, ix) + radial( it, iv)
					else
						radial( it, iv) = radial( it, iv) + timespace( it, ix)
					end if
				end if
			end do
		end do
	end subroutine radial_trace_tinterp
!---------------------------------------------------------------------------
	subroutine radial_trace_xinterp( adj, timespace, radial )
		integer                       :: it, iv, ix0, ix1
		real                          :: t, x, v, c0, c1
		real, dimension(:,:), pointer :: timespace, radial
		logical                       :: adj
		
		do it = 1,nt
			t = (it-1)*dt + t0
			do iv = 1,nv
				v = (iv-1)*dv + vmin
				x = v*t
				ix0 = floor((x-x0)/dx);  ix1 = ix0+1 
				if( ix1 <= nx .and. ix0 > 0 ) then
					c1 = (x-x0)/dx - ix0;  c0 = 1.0 - c1
					if( adj) then
						timespace( it, ix0) = c0*radial( it,iv) + timespace( it,ix0)
						timespace( it, ix1) = c1*radial( it,iv) + timespace( it,ix1)
					else
						radial( it, iv) = c0*timespace( it, ix0) + c1*timespace( it, ix1)
					end if
				end if
			end do
		end do
	end subroutine radial_trace_xinterp
!---------------------------------------------------------------------------
end module radial_trace_mod

